@@ -2,6 +2,7 @@ import io
 import json
 import logging
 import os
+from collections import OrderedDict
 
 import protoparser
 from django.apps import apps
@@ -37,7 +38,7 @@ class RegistryToProtoGenerator:
             )
             proto_by_app[app_name] = self.get_proto(app_name, registered_items)
 
-        return proto_by_app
+        return OrderedDict(sorted(proto_by_app.items()))
 
     def get_proto_path_for_app_name(self, app_name):
         return os.path.join(apps.get_app_config(app_name).path, "grpc", f"{app_name}.proto")
@@ -50,12 +51,16 @@ class RegistryToProtoGenerator:
         self._writer.write_line(f"package {self.project_name}.{app_name};")
         self._writer.write_line("")
         self._writer.write_line("IMPORT_PLACEHOLDER")
-        for grpc_controller_name, grpc_methods in registered_items[
-            "registered_controllers"
-        ].items():
-            self._generate_controller(grpc_controller_name, grpc_methods)
+        for grpc_controller_name, grpc_methods in sorted(
+            registered_items["registered_controllers"].items()
+        ):
+            self._generate_controller(
+                grpc_controller_name, OrderedDict(sorted(grpc_methods.items()))
+            )
 
-        for grpc_message_name, grpc_message in registered_items["registered_messages"].items():
+        for grpc_message_name, grpc_message in sorted(
+            registered_items["registered_messages"].items()
+        ):
             self._generate_message(grpc_message_name, grpc_message)
 
         return self._writer.get_code()
@@ -117,7 +122,7 @@ class RegistryToProtoGenerator:
             )
 
             # Info - AM - 30/04/2021 - Write all fields as defined in the serializer. Field_name is the name of the field ans field_type the instance of the drf field: https://www.django-rest-framework.org/api-guide/fields
-            for field_name, proto_type in grpc_message_fields:
+            for field_name, proto_type, comment in grpc_message_fields:
 
                 self.print(f"GENERATE FIELD: {field_name}", 4)
                 number += 1
@@ -127,10 +132,21 @@ class RegistryToProtoGenerator:
                 if "google.protobuf.Struct" in proto_type:
                     self._writer.import_struct = True
 
+                # Info - AM - 30/04/2021 - add comment
+                if comment:
+                    if type(comment) is list:
+                        for part_of_comment in comment:
+                            self.write_comment_line(part_of_comment)
+                    else:
+                        self.write_comment_line(comment)
+
                 self._writer.write_line(f"{proto_type} {field_name} = {number};")
         self._writer.write_line("}")
         self._writer.write_line("")
 
+    def write_comment_line(self, comment):
+        self._writer.write_line(f"//{comment}")
+
     def find_existing_number_for_field(self, grpc_message_name, field_name):
         """
         Find if the field for this grpc message was already existing and return its number
